//!HOOK MAIN
//!BIND HOOKED
//!DESC HLG to PQ

// Constants from SMPTE ST 2084-2014
const float pq_m1 = 0.1593017578125;    // ( 2610.0 / 4096.0 ) / 4.0;
const float pq_m2 = 78.84375;           // ( 2523.0 / 4096.0 ) * 128.0;
const float pq_c1 = 0.8359375;          // ( 3424.0 / 4096.0 ) or pq_c3 - pq_c2 + 1.0;
const float pq_c2 = 18.8515625;         // ( 2413.0 / 4096.0 ) * 32.0;
const float pq_c3 = 18.6875;            // ( 2392.0 / 4096.0 ) * 32.0;

const float pq_C  = 10000.0;

// Converts from linear cd/m^2 to the non-linear perceptually quantized space
// Note that this is in float, and assumes normalization from 0 - 1
// (0 - pq_C for linear) and does not handle the integer coding in the Annex
// sections of SMPTE ST 2084-2014
float Y_2_ST2084(float C) {
    // Note that this does NOT handle any of the signal range
    // considerations from 2084 - this returns full range (0 - 1)
    float L = C / pq_C;
    float Lm = pow(L, pq_m1);
    float N = (pq_c1 + pq_c2 * Lm) / (1.0 + pq_c3 * Lm);
    N = pow(N, pq_m2 );
    return N;
}

// converts from linear cd/m^2 to PQ code values
vec3 Y_2_ST2084_f3(vec3 L) {
    return vec3(Y_2_ST2084(L.r), Y_2_ST2084(L.g), Y_2_ST2084(L.b));
}

// Conversion of HLG to PQ signal, as detailed in Section 7 of ITU-R BT.2390-0
vec3 HLG_2_ST2084_1000nits(vec3 HLG)
{
    const float L_w   = 1000;
    const float L_b   = 0;
    const float alpha = L_w-L_b;
    const float beta  = L_b;
    const float gamma = 1.2;

    const float a = 0.17883277;
    const float b = 0.28466892; // 1.-4.*a;
    const float c = 0.55991073; // 0.5-a*log(4.*a);

    // HLG EOTF (non-linear signal value to display linear)
    // HLG to scene-linear
    vec3 sceneLinear = vec3(
        HLG.r >= 0 && HLG.r <= 0.5 ? pow(HLG.r, 2) / 3 : (exp((HLG.r - c) / a) + b) / 12,
        HLG.g >= 0 && HLG.g <= 0.5 ? pow(HLG.g, 2) / 3 : (exp((HLG.g - c) / a) + b) / 12,
        HLG.b >= 0 && HLG.b <= 0.5 ? pow(HLG.b, 2) / 3 : (exp((HLG.b - c) / a) + b) / 12
    );

    float Y_s = dot(sceneLinear, vec3(0.2627, 0.6780, 0.0593));

    // Scene-linear to display-linear
    vec3 displayLinear = alpha * pow(Y_s, gamma - 1) * sceneLinear + beta;

    // ST.2084 Inverse EOTF
    vec3 PQ = Y_2_ST2084_f3(displayLinear);

    return PQ;
}

vec4 color = HOOKED_texOff(vec2(0.0, 0.0));
vec4 hook() {
    color = max(color, 0.0);
    color.rgb = HLG_2_ST2084_1000nits(color.rgb);
    return color;
}
