//!HOOK MAIN
//!BIND HOOKED
//!DESC ACES Filmic Tone Mapping (xyY)

vec3 RGB_to_XYZ(vec3 RGB) {
    const mat3 M = mat3(
        0.6370, 0.1446, 0.1689,
        0.2627, 0.6780, 0.0593,
        0.0000, 0.0281, 1.0610);
    return M * RGB;
}

vec3 XYZ_to_RGB(vec3 XYZ) {
    const mat3 M = mat3(
         1.7167, -0.3557, -0.2534,
        -0.6667,  1.6165,  0.0158,
         0.0176, -0.0428,  0.9421);
    return M * XYZ;
}

// Transformations between CIE XYZ tristimulus values and CIE x,y
// chromaticity coordinates
vec3 XYZ_to_xyY(vec3 XYZ) {
    float divisor = (XYZ.x + XYZ.y + XYZ.z);
    if (divisor == 0) divisor = 1e-10;

    const float x = XYZ.x / divisor;
    const float y = XYZ.y / divisor;
    const float Y = XYZ.y;

    return vec3(x, y, Y);
}

vec3 xyY_to_XYZ(vec3 xyY) {
    const float x = xyY[0];
    const float y = xyY[1];
    const float Y = xyY[2];

    const float X = x * Y / max(y, 1e-10);
    const float Z = (1.0 - x - y) * Y / max(y, 1e-10);

    return vec3(X, Y, Z);
}

float aces(float x) {
    const float A = 2.51;
    const float B = 0.03;
    const float C = 2.43;
    const float D = 0.59;
    const float E = 0.14;
    return (x * (A * x + B)) / (x * (C * x + D) + E);
}

vec4 color = HOOKED_texOff(vec2(0.0, 0.0));
vec4 hook() {
    color.xyz = RGB_to_XYZ(color.rgb);
    color.xyz = XYZ_to_xyY(color.xyz);
    color.z   = aces(color.z);
    color.xyz = xyY_to_XYZ(color.xyz);
    color.rgb = XYZ_to_RGB(color.xyz);
    return color;
}
