//!HOOK MAIN
//!BIND HOOKED
//!DESC ITU-R BT.2446 Conversion Method C Tone Mapping

vec3 RGB_to_XYZ(vec3 RGB) {
    const mat3 M = mat3(
        0.6370, 0.1446, 0.1689,
        0.2627, 0.6780, 0.0593,
        0.0000, 0.0281, 1.0610);
    return M * RGB;
}

vec3 XYZ_to_RGB(vec3 XYZ) {
    const mat3 M = mat3(
         1.7167, -0.3557, -0.2534,
        -0.6667,  1.6165,  0.0158,
         0.0176, -0.0428,  0.9421);
    return M * XYZ;
}

// Transformations between CIE XYZ tristimulus values and CIE x,y
// chromaticity coordinates
vec3 XYZ_to_xyY(vec3 XYZ) {
    float divisor = (XYZ.x + XYZ.y + XYZ.z);
    if (divisor == 0) divisor = 1e-10;

    const float x = XYZ.x / divisor;
    const float y = XYZ.y / divisor;
    const float Y = XYZ.y;

    return vec3(x, y, Y);
}

vec3 xyY_to_XYZ(vec3 xyY) {
    const float x = xyY[0];
    const float y = xyY[1];
    const float Y = xyY[2];

    const float X = x * Y / max(y, 1e-10);
    const float Z = (1.0 - x - y) * Y / max(y, 1e-10);

    return vec3(X, Y, Z);
}

vec3 chroma_correction(vec3 x) {
    // Optional processing of chroma correction above HDR Reference White
    return x;
}

float bt2446c(float Y) {
    // HLG parameters
    // const float ip = 58.5 / k1;
    // const float k1 = 0.83802;
    // const float k3 = 0.74204;
    // const float k2 = 15.09968;
    // const float k4 = 78.99439;

    const float ip = 0.49261; // 100 / 203
    const float k1 = 0.83802;
    const float k3 = 0.74204;

    const float k2 = k1 * (1 - k3) * ip;
    const float k4 = (k1 * ip) - (k2 * log(1 - k3));
    return Y >= ip ? log((Y / ip) - k3) * k2 + k4 : k1 * Y;
}

vec4 color = HOOKED_texOff(vec2(0.0, 0.0));
vec4 hook() {
    color.xyz = RGB_to_XYZ(color.rgb);
    color.xyz = XYZ_to_xyY(color.xyz);
    color.xyz = chroma_correction(color.xyz);
    color.z   = bt2446c(color.z);
    color.xyz = xyY_to_XYZ(color.xyz);
    color.rgb = XYZ_to_RGB(color.xyz);
    return color;
}
